package com.mortennobel.openglinfo;

import javax.jnlp.DownloadService;
import javax.jnlp.DownloadServiceListener;
import javax.jnlp.ServiceManager;
import javax.jnlp.UnavailableServiceException;
import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.*;
import java.util.regex.Pattern;

public class OpenGLInfoPanel extends JFrame {
    private JTable table = new JTable();
    private JScrollPane scrollPane = new JScrollPane(table);
    private JCheckBox allExtensions = new JCheckBox("Show All OpenGL extensions", false);

    private JEditorPane editorPane = new JEditorPane();
    private JScrollPane editorPaneScroll = new JScrollPane(editorPane);
    private final String glVersion;
    private final String glRenderer;
    private final String glVendor;
    private final String glShadingLanguageVersion;
    private final String glExtensions;

    private OpenGLExtensionInfo infoObj;
    private Map<String, String> info;
    private Vector<Vector<String>> tableData = new Vector<Vector<String>>();


    public OpenGLInfoPanel(String glVersion, String glRenderer, String glVendor, String glShadingLanguageVersion, String glExtensions) {
        this.glVersion = glVersion;
        this.glRenderer = glRenderer;
        this.glVendor = glVendor;
        this.glShadingLanguageVersion = glShadingLanguageVersion;
        this.glExtensions = glExtensions;
        setTitle("OpenGL Info (www.nobel-joergensen.com)");
        setSize(700,700);
        setVisible(true);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JPanel topPanel = new JPanel(new BorderLayout());
        topPanel.add(allExtensions, BorderLayout.NORTH);
        topPanel.add(scrollPane, BorderLayout.CENTER);

        editorPaneScroll.setBorder(new TitledBorder("OpenGL Extension detail"));

        // create top component
        getContentPane().setLayout(new GridLayout(0,1));
        getContentPane().add(topPanel);
        getContentPane().add(editorPaneScroll);

        try {
            InputStream is = getClass().getResourceAsStream("filedb.ser");
            ObjectInputStream ois = new ObjectInputStream(is);
            infoObj = (OpenGLExtensionInfo) ois.readObject();
            info = infoObj.getlAllExtensions();
            is.close();
        } catch (Exception e) {
            e.printStackTrace();
            editorPane.setText("Error reading extension info from webpage "+OpenGLExtensionInfo.webUrl);
        }

        updateTable();

        allExtensions.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                updateTable();
            }
        });

        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                if (table.getSelectedRow()==-1){
                    return;
                }
                String extensionName = tableData.get(table.getSelectedRow()).get(0);
                if (info.containsKey(extensionName)){
                    editorPane.setText("Loading "+extensionName+" ...");
                    final String url = info.get(extensionName);
                    final String content = infoObj.getSpec(extensionName);
                    try {
                        editorPane.setText(content);
                        editorPane.setCaretPosition(0);
                    } catch (Exception e1) {
                        editorPane.setText("Unable to load "+url);
                        e1.printStackTrace();
                    }
                } else {
                    editorPane.setText("Not found");
                }
            }
        });
    }

    private void updateTable() {
        Vector<String> columnNames = new Vector<String>(Arrays.asList("Name", "Description", "Enabled"));
        tableData.clear();
        tableData.add(new Vector<String>(Arrays.asList("glVersion", glVersion, "True")));
        tableData.add(new Vector<String>(Arrays.asList("glRenderer", glRenderer, "True")));
        tableData.add(new Vector<String>(Arrays.asList("glVendor", glVendor, "True")));
        tableData.add(new Vector<String>(Arrays.asList("glShadingLanguageVersion", glShadingLanguageVersion, "True")));
        String[] allExtensions = glExtensions.split(" ");
        for (String extensionName : allExtensions){
            tableData.add(new Vector<String>(Arrays.asList(extensionName, " ", "True")));
        }
        if (this.allExtensions.isSelected()){
            Set<String> enabledExtensions = new HashSet<String>(Arrays.asList(allExtensions));
            for (String extensionName : info.keySet()){
                if (!enabledExtensions.contains(extensionName)){
                    tableData.add(new Vector<String>(Arrays.asList(extensionName, " ", "False")));
                }
            }
        }
        TableModel model = new DefaultTableModel(tableData,columnNames);
        table.setModel(model);
        invalidate();
        validate();
    }
}
