package com.mortennobel.openglinfo;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class OpenGLExtensionInfo implements Serializable {
    static private final long serialVersionUID = 42L;

    public static final String webUrl = "http://www.opengl.org/registry/";

    public Map<String,String> allExtensionsList;
    public Map<String, byte[]> extentionSpec = new HashMap<String,byte[]>();

    /**
     * Factory class
     */
    private OpenGLExtensionInfo (){}
    /**
     *
     * @return Map<ExtensionName, spec url>
     */
    public Map<String,String> getlAllExtensions() throws Exception {
        if (allExtensionsList !=null){
            return allExtensionsList;
        }
        allExtensionsList = new TreeMap<String,String>();

        InputStream is = null;
        HttpURLConnection urlCon = null;
        try {
            URL url = new URL(webUrl);
            urlCon = (HttpURLConnection) url.openConnection();
            urlCon.connect();

            is = urlCon.getInputStream();

            InputStreamReader isr = new InputStreamReader(is, "UTF-8");
            BufferedReader br = new BufferedReader(isr);
            Pattern pattern = Pattern.compile("<a href=\"(specs[^\"]*)\">([^<]*)</a");
            String line;
            while ((line=br.readLine())!=null){
                Matcher m = pattern.matcher(line);
                if (m.find()){
                    String urlLink = "http://www.opengl.org/registry/"+m.group(1);
                    String name = m.group(2);
                    allExtensionsList.put(name,urlLink);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (is!=null){
                is.close();
            }
            if (urlCon!=null){
                urlCon.disconnect();
            }
        }
        return allExtensionsList;
    }

    public String getSpec(String ext){
        byte[] gZipData = extentionSpec.get(ext);
        if (gZipData==null){
            System.out.println(ext+" not found");
        }
        ByteArrayInputStream bais = new ByteArrayInputStream(gZipData);
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        try {
            GZIPInputStream gIn = new GZIPInputStream(bais);
            byte[] buffer = new byte[512];
            int read;
            while ((read = gIn.read(buffer))>0){
                out.write(buffer,0,read);
            }
            return new String(out.toByteArray());
        } catch (IOException e) {
            e.printStackTrace();
            return "Error "+e.getMessage();
        }

    }

    private void populateExtentionSpec(){
        String date = new Date().toString();
        int counter = 0;
        for (String name: allExtensionsList.keySet()){
            String url = allExtensionsList.get(name);
            String content = readUrl(url)+"\nDownloaded from: "+url+" date "+date;
            ByteArrayOutputStream out = new ByteArrayOutputStream();
            try {
                GZIPOutputStream gOut = new GZIPOutputStream(out);
                gOut.write(content.getBytes());
                gOut.finish();
                extentionSpec.put(name,out.toByteArray());
            } catch (IOException e) {
                e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            }
            System.out.println((++counter)+"/"+allExtensionsList.size());
        }
    }

    public static String readUrl(String webUrl) {

        StringBuilder sb = new StringBuilder();

        InputStream is = null;
        HttpURLConnection urlCon = null;
        try {
            URL url = new URL(webUrl);
            urlCon = (HttpURLConnection) url.openConnection();
            urlCon.connect();

            is = urlCon.getInputStream();

            InputStreamReader isr = new InputStreamReader(is, "UTF-8");
            BufferedReader br = new BufferedReader(isr);
            String line;
            while ((line=br.readLine())!=null){
                sb.append(line);
                sb.append("\n");
            }
        } catch (Exception e) {
            e.printStackTrace();
            sb.append("Error reading "+webUrl+" "+e.getMessage());
        } finally {
            if (is!=null){
                try {
                    is.close();
                } catch (IOException e) {
                    // ignore
                }
            }
            if (urlCon!=null){
                urlCon.disconnect();
            }
        }
        return sb.toString();
    }

    public static void main(String[] args) {
        if (args.length!=1){
            System.out.println("Usage java OpenGLExtensionInfo [dest serialized file]");
            System.exit(1);
        }
        System.out.println("Dest "+args[0]);
        OpenGLExtensionInfo info = new OpenGLExtensionInfo();
        try {
            info.getlAllExtensions();
            info.populateExtentionSpec();
            FileOutputStream fos = new FileOutputStream(args[0]);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
            oos.writeObject(info);
            oos.close();
            fos.close();
        } catch (Exception e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
    }


}
